from .. import utils

KEY = 'wikipedia'
TEST_KEY = KEY
DESCRIPTION = 'English wikipedia picture of the day, mostly about history'
URL = 'https://en.wikipedia.org/wiki/Wikipedia:Picture_of_the_day'

def pre_process():
	return None

def process(date):
	soup = utils.get_soup_from_url(URL)
	imgs = utils.find_tags_from_soup(soup, "img")
	current_width = imgs[0].get('width')
	full_width = imgs[0].get('data-file-width')
	return "https:" + imgs[0].get('src').replace(str(current_width + "px"), str(full_width + "px"))
    
def post_process(image):
	return None


from src.websites import WEBSITES
from src.managers.managerfactory import ManagerFactory
from src.websites.websitefactory import WebsiteFactory
from src.log import logger
import traceback

def test():
    counter = 0
    succes = 0
    fails = {}
    logger.debug('Test in progress...')
    logger.debug('0/' + str(len(WEBSITES)) + '\tSucces: 0\tFails: 0')
    
    manager_factory = ManagerFactory()
    manager = manager_factory.get_manager()
    website_factory = WebsiteFactory()
    
    for website in WEBSITES.values():
        try :
            website = website_factory.get_website(website.TEST_KEY)
            manager.set(website, test=True)
            succes+=1
        except Exception as e:
            fails[website.KEY] = str(traceback.format_exc())
        
        counter+=1
        logger.debug(str(counter) + '/' + str(len(WEBSITES)) + '\tSucces: ' + str(succes) + '\tFails: ' + str(len(fails)), CR=True, flush=True)

    if(len(fails) == 0):
        logger.debug('OK')
    else:
        for fail in fails.keys():
            logger.debug('=============')
            logger.debug(fail + " : " + fails[fail])

if __name__ == "__main__":
	try:
		test()
	except KeyboardInterrupt:
		None


from .. import utils
from ..exceptions import WallmeException

KEY = 'apod'
TEST_KEY = KEY
DESCRIPTION = 'Astronomy Picture Of the Day'
URL = 'https://apod.nasa.gov/apod/astropix.html'

def pre_process():
	return None
    
def process(date):
    try:
        soup = utils.get_soup_from_url(URL)
        imgs = utils.find_tags_from_soup(soup, "img")
        return 'https://apod.nasa.gov/apod/' + imgs[0].get('src')
    except:
        raise WallmeException('No picture found today. It could be a video.') 
    
def post_process(image):
	return None


from .. import utils

KEY = 'bing'
TEST_KEY = KEY
DESCRIPTION = 'Various landscapes taken from the web search engine Bing'
URL = 'http://www.bing.com'

def pre_process():
	return None
    
def process(date):
	json = utils.get_json_from_url('http://www.bing.com/HPImageArchive.aspx?format=js&idx=0&n=1&mkt=EN-IN')
	return "https://www.bing.com" + json['images'][0]['url']
    
def post_process(image):
	return None


import traceback, argparse
from .websites import WEBSITES
from .websites.websitefactory import WebsiteFactory
from .managers.managerfactory import ManagerFactory
from .exceptions import WallmeException

def main():

	parser = argparse.ArgumentParser(description='Change your desktop wallpaper based on websites.')
	parser.add_argument('-list', action='store_true', help='list all the available websites')
	parser.add_argument('-info', type=str, help='open the webpage on which the image is taken from')
	parser.add_argument('-set', type=str, help='change the wallpaper')
	parser.add_argument('-set-startup', type=str, help='change your wallpaper on startup')
	parser.add_argument('-unset-startup', action='store_true', help='stop changing your wallpaper on startup')
	args = parser.parse_args()
    
	try:
		if(args.list):
			for key in WEBSITES.keys():
				print(key + " - " + WEBSITES[key].DESCRIPTION)
		else:
			manager_factory = ManagerFactory()
			manager = manager_factory.get_manager()
        
			if(args.unset_startup):
				manager.unset_startup()
			else:
				website_factory = WebsiteFactory()
				if(args.set):
					website = website_factory.get_website(args.set)
					manager.set(website)
				if(args.set_startup):
					website = website_factory.get_website(args.set_startup)
					manager.set_startup(website)
				if(args.info):
					website = website_factory.get_website(args.info)
					manager.info(website)
    
	except WallmeException as e:
		print(e)
	except Exception as e:
		traceback.print_exc()
		print("=========\nPlease, report this issue : https://github.com/LucBerge/wallme/issues")

if __name__ == "__main__":
	try:
		main()
	except KeyboardInterrupt:
		None


from .. import utils

KEY = 'the-guardian'
TEST_KEY = KEY
DESCRIPTION = 'Each day the picture editor of the Guardian brings you a selection of photo highlights'
URL = 'https://www.theguardian.com/news/series/ten-best-photographs-of-the-day'

def pre_process():
	return None

def process(date):
	soup = utils.get_soup_from_url(URL)
	imgs = utils.find_tags_from_soup(soup, "a", attributes={"class": "u-faux-block-link__overlay"})
	soup = utils.get_soup_from_url(imgs[0].get('href'))
	sources = utils.find_tags_from_soup(soup, "source", attributes={"sizes": "1900px"})
	return sources[0].get('srcset').split(' ')[0]
    
def post_process(image):
	return None


KEY = 'tetras'
TEST_KEY = KEY
DESCRIPTION = 'Amateur pictures of the Alpes and the Dauphine'
URL = 'http://tetras.org/Semaine.jpg'

def pre_process():
	return None
    
def process(date):
	return URL

def post_process(image):
	return None


from . import WEBSITES
from ..exceptions import WallmeException

class WebsiteFactory():

    def get_website(self, key):
    
        dot_index = key.find('.')
        if dot_index > 0:
            subkey = key[dot_index+1:]
            key = key[:dot_index]
    
        if(key not in WEBSITES.keys()):
            raise WallmeException("Website '" + key + "' not supported. Use the -list option to list all the posibilities.")
        
        if(dot_index > 0):
            WEBSITES[key].subkey = subkey
            
        return WEBSITES[key]


import requests, json
from bs4 import BeautifulSoup
from .exceptions import WallmeException

headers = {'User-agent': 'wallme'}

def get_webpage_from_url(url):
    try:
        webpage = requests.get(url, headers=headers)
    except requests.exceptions.ConnectionError as e:
        raise WallmeException("Cannot retrieve webpage '" + url + "'. Make sure you have an internet connection, retry later.")
    
    if webpage.status_code == 429:
        raise WallmeException("Cannot retrieve webpage '" + url + "' (error " + str(webpage.status_code) + "). Too many requests, retry later.")
    if webpage.status_code != 200:
        raise Exception("Cannot retrieve webpage '" + url + "' (error " + str(webpage.status_code) + "). Please make sure the url is valid.")
    return webpage.text

def get_json_from_url(url):
    return json.loads(get_webpage_from_url(url))
    
def get_soup_from_url(url, parser='html.parser'):
    return BeautifulSoup(get_webpage_from_url(url), parser)

def find_tags_from_soup(soup, tag, attributes=None):
    if(attributes):
        tags = soup.findAll(tag, attributes)
    else:
        tags = soup.find_all(tag)
 
    if(len(tags) == 0):
        raise Exception("Couldn't find tag '" + tag + "' with attributes '" + str(attributes) + "'.")
    return tags


import types

from . import apod
from . import astrobin
from . import bing
from . import epod
from . import nasa
from . import nationalgeographic
from . import reddit
from . import tetras
from . import theguardian
from . import thetelegraph
from . import wikipedia

WEBSITES = {}

for item in list(globals().items()):
	if(isinstance(item, tuple)):
		if(isinstance(item[1], types.ModuleType)):
			if("websites." in item[1].__name__):
				WEBSITES[item[1].KEY] = item[1]


from .. import utils
from ..exceptions import WallmeException

KEY = 'reddit'
TEST_KEY = 'reddit.earthporn'
DESCRIPTION = 'Top pictures from the topic of your choice. Use reddit.<topic> as the key. (e.g. : reddit.earthporn)'
URL = 'http://www.reddit.com/r/'

def pre_process():
    global URL
    try:
    	URL = URL + subkey
    except:
    	raise WallmeException("Missing subkey : Please use 'reddit.<subkey>'.")
    return None
    
def process(date):
    json = utils.get_json_from_url('http://www.reddit.com/r/' + subkey + '/top/.json?limit=1')
    return json['data']['children'][0]['data']['url']
    
def post_process(image):
    return None


import platform
from ..exceptions import WallmeException
from .linux import Linux
from .windows import Windows

class ManagerFactory():

    def get_manager(self):
        system = platform.system()
        if(system == "Linux"):
            return Linux()
        elif(system == "Windows"):
            return Windows()
        else:
            raise WallmeException("Unknown system : '" + system + "'")


from .. import utils
from PIL import Image

KEY = 'astrobin'
TEST_KEY = KEY
DESCRIPTION = 'Pictures of Galaxies, Stars and Planets'
URL = 'https://www.astrobin.com/iotd/archive/'

def pre_process():
    return None

def process(date):
	soup = utils.get_soup_from_url(URL)
	divs = utils.find_tags_from_soup(soup, "div", attributes={"class": "astrobin-image-container"})
	soup = utils.get_soup_from_url("https://www.astrobin.com/full" + divs[0].a.get('href') + "/0")
	imgs = utils.find_tags_from_soup(soup, "img", attributes={"class": "astrobin-image"})
	return imgs[0].get('src')

def post_process(image):
	with Image.open(image) as img:
		width, height = img.size
		if(width < height):
			img = img.transpose(Image.ROTATE_90)
			img.save(image)
	


import os
from .manager import Manager
from ..exceptions import WallmeException

class Linux(Manager):
        
    SERVICE_FILE = "/lib/systemd/system/wallme.service"

    def set(self, website, test=False):
        super().download(website)
        if(not test):
            if os.system("/usr/bin/gsettings set org.gnome.desktop.background picture-uri " + self.IMAGE) != 0:
                raise WallmeException("Cannot set wallpaper")
            
    def set_startup(self, website):
        
        self.unset_startup()
        
        try:
            fullkey = website.KEY + '.' + website.subkey
        except:
            fullkey = website.KEY

        user = os.popen("who | awk 'FNR == 1 {print $1}'").read().rstrip()
        
        if(not os.path.exists(self.SERVICE_FILE)):
            with open(self.SERVICE_FILE, "wt") as f:
                f.write("[Unit]\n\
Description=Wallme service.\n\
After=network-online.target\n\
\n\
[Service]\n\
Type=simple\n\
User=" + user + "\n\
Group=" + user + "\n\
ExecStart=/usr/local/bin/wallme -set " + fullkey + "\n\
\n\
[Install]\n\
WantedBy=multi-user.target")

        os.system("sudo systemctl enable wallme")

    def unset_startup(self):
        if(os.path.exists(self.SERVICE_FILE)):
            os.remove(self.SERVICE_FILE)
            os.system("sudo systemctl disable wallme")


import requests
from datetime import date
from pathlib import Path
import webbrowser

class Manager():

    IMAGE = str(Path.home()) + '/wallme.jpg'

    def download(self, website):
    
        #Pre process
        pre_process_result = website.pre_process()
        
        #Get image url
        image_url = website.process(date.today())

        #Download image
        img_data = requests.get(image_url).content
        with open(self.IMAGE, 'wb') as handler:
            handler.write(img_data)

        #Post process
        post_process_result = website.post_process(self.IMAGE)

    def info(self, website):
        #Pre process
        pre_process_result = website.pre_process()
        
        #Open the browser
        webbrowser.open(website.URL, new=2)


from .. import utils

KEY = 'nasa'
TEST_KEY = KEY
DESCRIPTION = 'Pictures related to nasa\'s missions'
URL = 'https://www.nasa.gov/multimedia/imagegallery/iotd.html'

def pre_process():
	return None

def process(date):
	json = utils.get_json_from_url('https://www.nasa.gov/api/2/ubernode/_search?size=1&from=0&sort=promo-date-time%3Adesc&q=((ubernode-type%3Aimage)%20AND%20(routes%3A1446))&_source_include=promo-date-time%2Cmaster-image%2Cnid%2Ctitle%2Ctopics%2Cmissions%2Ccollections%2Cother-tags%2Cubernode-type%2Cprimary-tag%2Csecondary-tag%2Ccardfeed-title%2Ctype%2Ccollection-asset-link%2Clink-or-attachment%2Cpr-leader-sentence%2Cimage-feature-caption%2Cattachments%2Curi')
	uri = json['hits']['hits'][0]['_source']['master-image']['uri']
	return "https://www.nasa.gov/sites/default/files/styles/full_width_feature/public/" + uri.replace("public://","")

def post_process(image):
	return None


import ctypes, os
try:
    import winreg
except:
    pass
from .manager import Manager
from ..exceptions import WallmeException

class Windows(Manager):

    REGISTRY_KEY = "wallme"

    def set(self, website, test=False):
        super().download(website)
        if(not test):
            if ctypes.windll.user32.SystemParametersInfoW(20, 0, self.IMAGE, 3) != 1:
                raise WallmeException("Cannot set wallpaper")

    def set_startup(self, website):
        try:
            fullkey = website.KEY + '.' + website.subkey
        except:
            fullkey = website.KEY
            
        reg_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Run', 0, winreg.KEY_SET_VALUE)
        with reg_key:
            winreg.SetValueEx(reg_key, self.REGISTRY_KEY, 0, winreg.REG_SZ, "cmd /c start /min wallme.exe -set " + fullkey)
            
    def unset_startup(self):
        reg_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Run', 0, winreg.KEY_SET_VALUE)
        with reg_key:
            winreg.DeleteValue(reg_key, self.REGISTRY_KEY)


from .. import utils

KEY = 'the-telegraph'
TEST_KEY = KEY
DESCRIPTION = 'Pictures reflecting the political, social or environmental situations in the world'
URL = 'https://www.telegraph.co.uk/news/pictures/'

def pre_process():
	return None
    
def process(date):
	soup = utils.get_soup_from_url(URL)
	h3s = utils.find_tags_from_soup(soup, "h3", attributes={"class": "u-heading-6"})
	soup = utils.get_soup_from_url("https://www.telegraph.co.uk" + h3s[0].a.get('href'))
	imgs = utils.find_tags_from_soup(soup, "img", attributes={"class": "gallery__regwall-image"})
	return "https://www.telegraph.co.uk" + imgs[0].get('src').split('?')[0]

def post_process(image):
	return None


# coding: utf8

class bcolors:
    OK = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'
    CR = '\033[F'

class Log(object):
    def __init__(self, name):
        self.name = name

    def debug(self, msg, CR=False, flush=False):
        if(CR==True):
            print(bcolors.CR + "[" + self.name + "] " + msg, flush=flush)
        else:
            print("[" + self.name + "] " + msg, flush=flush)

    def warning(self, msg, flush=False):
        print("[" + self.name + "] " + bcolors.WARNING + "WARNING : " + bcolors.END + msg, flush=flush)

    def error(self, msg, flush=False):
        print("[" + self.name + "] " + bcolors.FAIL + "ERROR : " + bcolors.END + msg, flush=flush)

logger = Log("wallme")